/*
 * @(#)src/classes/sov/java/util/HashMap.java, util, asdev, 20080304 1.36
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 * IBM SDK, Java(tm) 2 Technology Edition, v5.0
 * (C) Copyright IBM Corp. 1998, 2005. All Rights Reserved
 * ===========================================================================
 */

/*
 * ===========================================================================
 (C) Copyright Sun Microsystems Inc, 1992, 2004. All rights reserved.
 * ===========================================================================
 */

/*
 * @(#)HashMap.java    1.63 04/02/19
 *
 */

/*
*
* Change activity:
*
* Reason Date   Origin   Description
* ------ ------ -------- ---------------------------------------------------
* 60126.1 160403 kiranram Protect transfer() from concurrent use
* 84391.1 230505 stalleyj Add missing part of fix from 60126
* 95761   210905 niced    Sunbug 2121884: Improve resilience of HashMap and ArrayList
* 098538  131205 riclau   Reorder get() to avoid JIT optimization problem
* 098545  131205 riclau   Eliminate NULL_KEY
* 098546  131205 riclau   Modify hash function
* 116438  270207 nmasson  Performance improvements
* 117168  130307 nmasson  Revert the above fix
* 117216  140307 nikolag  Integer key specialization
* 120435  040607 nmasson  Bug fix re above
* 121423  150607 mbluemel Make it possible to get back the original key
*                         for Integer keys, rather than recreating it
* 121735  280607 andhall  Fix for 121423 - fixed size 1 map support and 
*                         corrected removeMappingInt
* 124312  030907 mbluemel Sun Regression test failed, due to missing some
*                         Integer handling in Entry.equals()
* ===========================================================================
*/

package java.util;
import  java.io.*;

/**
 * Hash table based implementation of the <tt>Map</tt> interface.  This
 * implementation provides all of the optional map operations, and permits
 * <tt>null</tt> values and the <tt>null</tt> key.  (The <tt>HashMap</tt>
 * class is roughly equivalent to <tt>Hashtable</tt>, except that it is
 * unsynchronized and permits nulls.)  This class makes no guarantees as to
 * the order of the map; in particular, it does not guarantee that the order
 * will remain constant over time.
 *
 * <p>This implementation provides constant-time performance for the basic
 * operations (<tt>get</tt> and <tt>put</tt>), assuming the hash function
 * disperses the elements properly among the buckets.  Iteration over
 * collection views requires time proportional to the "capacity" of the
 * <tt>HashMap</tt> instance (the number of buckets) plus its size (the number
 * of key-value mappings).  Thus, it's very important not to set the initial
 * capacity too high (or the load factor too low) if iteration performance is
 * important.
 *
 * <p>An instance of <tt>HashMap</tt> has two parameters that affect its
 * performance: <i>initial capacity</i> and <i>load factor</i>.  The
 * <i>capacity</i> is the number of buckets in the hash table, and the initial
 * capacity is simply the capacity at the time the hash table is created.  The
 * <i>load factor</i> is a measure of how full the hash table is allowed to
 * get before its capacity is automatically increased.  When the number of
 * entries in the hash table exceeds the product of the load factor and the
 * current capacity, the capacity is roughly doubled by calling the
 * <tt>rehash</tt> method.
 *
 * <p>As a general rule, the default load factor (.75) offers a good tradeoff
 * between time and space costs.  Higher values decrease the space overhead
 * but increase the lookup cost (reflected in most of the operations of the
 * <tt>HashMap</tt> class, including <tt>get</tt> and <tt>put</tt>).  The
 * expected number of entries in the map and its load factor should be taken
 * into account when setting its initial capacity, so as to minimize the
 * number of <tt>rehash</tt> operations.  If the initial capacity is greater
 * than the maximum number of entries divided by the load factor, no
 * <tt>rehash</tt> operations will ever occur.
 *
 * <p>If many mappings are to be stored in a <tt>HashMap</tt> instance,
 * creating it with a sufficiently large capacity will allow the mappings to
 * be stored more efficiently than letting it perform automatic rehashing as
 * needed to grow the table.
 *
 * <p><b>Note that this implementation is not synchronized.</b> If multiple
 * threads access this map concurrently, and at least one of the threads
 * modifies the map structurally, it <i>must</i> be synchronized externally.
 * (A structural modification is any operation that adds or deletes one or
 * more mappings; merely changing the value associated with a key that an
 * instance already contains is not a structural modification.)  This is
 * typically accomplished by synchronizing on some object that naturally
 * encapsulates the map.  If no such object exists, the map should be
 * "wrapped" using the <tt>Collections.synchronizedMap</tt> method.  This is
 * best done at creation time, to prevent accidental unsynchronized access to
 * the map: <pre> Map m = Collections.synchronizedMap(new HashMap(...));
 * </pre>
 *
 * <p>The iterators returned by all of this class's "collection view methods"
 * are <i>fail-fast</i>: if the map is structurally modified at any time after
 * the iterator is created, in any way except through the iterator's own
 * <tt>remove</tt> or <tt>add</tt> methods, the iterator will throw a
 * <tt>ConcurrentModificationException</tt>.  Thus, in the face of concurrent
 * modification, the iterator fails quickly and cleanly, rather than risking
 * arbitrary, non-deterministic behavior at an undetermined time in the
 * future.
 *
 * <p>Note that the fail-fast behavior of an iterator cannot be guaranteed
 * as it is, generally speaking, impossible to make any hard guarantees in the
 * presence of unsynchronized concurrent modification.  Fail-fast iterators
 * throw <tt>ConcurrentModificationException</tt> on a best-effort basis. 
 * Therefore, it would be wrong to write a program that depended on this
 * exception for its correctness: <i>the fail-fast behavior of iterators
 * should be used only to detect bugs.</i>
 *
 * <p>This class is a member of the 
 * <a href="{@docRoot}/../guide/collections/index.html">
 * Java Collections Framework</a>.
 *
 * @author  Doug Lea
 * @author  Josh Bloch
 * @author  Arthur van Hoff
 * @author  Neal Gafter
 * @version 1.63, 02/19/04
 * @see     Object#hashCode()
 * @see     Collection
 * @see        Map
 * @see        TreeMap
 * @see        Hashtable
 * @since   1.2
 */

public class HashMap<K,V>
    extends AbstractMap<K,V>
    implements Map<K,V>, Cloneable, Serializable
{

    /**
     * The default initial capacity - MUST be a power of two.
     */
    static final int DEFAULT_INITIAL_CAPACITY = 16;

    /**
     * The maximum capacity, used if a higher value is implicitly specified
     * by either of the constructors with arguments.
     * MUST be a power of two <= 1<<30.
     */
    static final int MAXIMUM_CAPACITY = 1 << 30;

    /**
     * The load factor used when none specified in constructor.
     **/
    static final float DEFAULT_LOAD_FACTOR = 0.75f;

    /**
     * The table, resized as necessary. Length MUST Always be a power of two.
     */
    transient Entry[] table;

    /**
     * The number of key-value mappings contained in this identity hash map.
     */
    transient int size;
  
    /**
     * The next size value at which to resize (capacity * load factor).
     * @serial
     */
    int threshold;
  
    /**
     * The load factor for the hash table.
     *
     * @serial
     */
    final float loadFactor;

    /**
     * The number of times this HashMap has been structurally modified
     * Structural modifications are those that change the number of mappings in
     * the HashMap or otherwise modify its internal structure (e.g.,
     * rehash).  This field is used to make iterators on Collection-views of
     * the HashMap fail-fast.  (See ConcurrentModificationException).
     */
    transient volatile int modCount;
    
    // Start - ibm@60126.1

    /**
     * Contention flag for transfer() - meanings are :-
     *     0 - Free
     *     1 - Busy
     *     2 - Contended
     */
    private transient int contentionFlag = 0;

    /**
     * Entry and exit code for transfer()
     */
    private synchronized void onEntry() {
       switch(contentionFlag) {
           case(0):    contentionFlag=1;       /* Free -> Busy */
                       break;
           case(1):    contentionFlag=2;       /* Busy -> Contended */
                                               //FALLTHRU
           case(2):    throw new ConcurrentModificationException(
                           "concurrent access to HashMap attempted by " + Thread.currentThread());
           default:    throw new RuntimeException(
                           "Unexpected contentionFlag " + contentionFlag);
        }
    }
    
    private synchronized void onExit() {
        int oldContentionFlag=contentionFlag;
        contentionFlag=0;
        switch(oldContentionFlag) {
            case(1):    break;                                     /* Busy -> Free */
            case(2):    throw new ConcurrentModificationException( /* Contended -> Free */
                             "concurrent access to HashMap attempted by " + Thread.currentThread());
            default:    throw new RuntimeException(
                            "Unexpected contentionFlag " + oldContentionFlag);
        }
    }

    // End - ibm@60126.1



    /**
     * Constructs an empty <tt>HashMap</tt> with the specified initial
     * capacity and load factor.
     *
     * @param  initialCapacity The initial capacity.
     * @param  loadFactor      The load factor.
     * @throws IllegalArgumentException if the initial capacity is negative
     *         or the load factor is nonpositive.
     */
    public HashMap(int initialCapacity, float loadFactor) {
        if (initialCapacity < 0)
            throw new IllegalArgumentException("Illegal initial capacity: " +
                                               initialCapacity);
        if (initialCapacity > MAXIMUM_CAPACITY)
            initialCapacity = MAXIMUM_CAPACITY;
        if (loadFactor <= 0 || Float.isNaN(loadFactor))
            throw new IllegalArgumentException("Illegal load factor: " +
                                               loadFactor);

        // Find a power of 2 >= initialCapacity
        // Integer key optimisation algorithm doesn't work with size 1 hashtables
        // so minimum size is 2
        int capacity = 2;                                        /*ibm@121735*/
        while (capacity < initialCapacity) 
            capacity <<= 1;
    
        this.loadFactor = loadFactor;
        threshold = (int)(capacity * loadFactor);
        table = new Entry[capacity];
        init();
    }
  
    /**
     * Constructs an empty <tt>HashMap</tt> with the specified initial
     * capacity and the default load factor (0.75).
     *
     * @param  initialCapacity the initial capacity.
     * @throws IllegalArgumentException if the initial capacity is negative.
     */
    public HashMap(int initialCapacity) {
        this(initialCapacity, DEFAULT_LOAD_FACTOR);
    }

    /**
     * Constructs an empty <tt>HashMap</tt> with the default initial capacity
     * (16) and the default load factor (0.75).
     */
    public HashMap() {
        this.loadFactor = DEFAULT_LOAD_FACTOR;
        threshold = (int)(DEFAULT_INITIAL_CAPACITY * DEFAULT_LOAD_FACTOR);
        table = new Entry[DEFAULT_INITIAL_CAPACITY];
        init();
    }

    /**
     * Constructs a new <tt>HashMap</tt> with the same mappings as the
     * specified <tt>Map</tt>.  The <tt>HashMap</tt> is created with
     * default load factor (0.75) and an initial capacity sufficient to
     * hold the mappings in the specified <tt>Map</tt>.
     *
     * @param   m the map whose mappings are to be placed in this map.
     * @throws  NullPointerException if the specified map is null.
     */
    public HashMap(Map<? extends K, ? extends V> m) {
        this(Math.max((int) (m.size() / DEFAULT_LOAD_FACTOR) + 1,
                      DEFAULT_INITIAL_CAPACITY), DEFAULT_LOAD_FACTOR);
        putAllForCreate(m);
    }

    // internal utilities

    /**
     * Initialization hook for subclasses. This method is called
     * in all constructors and pseudo-constructors (clone, readObject)
     * after HashMap has been initialized but before any entries have
     * been inserted.  (In the absence of this method, readObject would
     * require explicit knowledge of subclasses.)
     */
    void init() {
    }

    /*ibm@98545 -- Remove NULL_KEY, maskNull and unmaskNull methods */

    /**
     * Returns a hash value for the specified object.
     */
    static int hash(Object x) {
        int h = (x == null ? 0 : x.hashCode()); /*ibm@98545*/

        /*ibm@98546 -- Analysis has shown just returning the hashCode 
         *             reduces hash collisions and chaining when run on J9.
         */

        /**
         * Sun's 1.5.0 update 9 version is a bit better
         */
        h ^= (h >>> 20) ^ (h >>> 12); 			/*ibm@117216*/
        return h ^ (h >>> 7) ^ (h >>> 4); 		/*ibm@117216*/
    }


    private static final int INT_KEY_FLAG = 1;
    /**
     * ibm@121423
     * Turns a hash into the reduced version we'll store<br>
     *
     * Following Tim Ellison's guidance, we exploit the fact
     * that all Entrys hung from a specific Hash bucket will
     * have the bucket number in low order "n" bits, where n
     * is the power of two corresponding to the table size.
     *
     * This means that we can use the rightmost bit to hold a
     * flag bit (INT_KEY_FLAG) to indicate that the key is
     * an Integer (cheaper than using instanceof on the key)
     *
     * ibm@124312
     * Made these methods "static" so the static inner class
     * "Entry" can use flagIntegerHash (changed all for
     * consistency)
     */
    static int flagNonIntegerHash(int h) {
        return h & ~INT_KEY_FLAG;
    }
    static int flagIntegerHash(int h) {
        return h | INT_KEY_FLAG;
    }

    /**
     * ibm@121423
     * recreates the original hash from the reduced version and
     * adds back the bit we lost. (Needed during resize ...)
     * Note: assumes that low bits of h are equivalent to
     * bucketnumber, except for the rightmost bit.
     *
     * ibm@124312
     * Made this methods "static" for consistency
     */
    static int unFlagHash(int h,int bucketNumber) {
        return (h & ~INT_KEY_FLAG) | bucketNumber;
    }

    /** 
     * Check for equality of non-null reference x and possibly-null y. 
     */
    static boolean eq(Object x, Object y) {
        return x == y || (x != null && x.equals(y)); /*ibm@98545 ibm@120435 ibm@121423*/
    }

    /**
     * Returns index for hash code h. 
     */
    static int indexFor(int h, int length) {
        return h & (length-1);
    }
 
    /**
     * Returns the number of key-value mappings in this map.
     *
     * @return the number of key-value mappings in this map.
     */
    public int size() {
        return size;
    }
  
    /**
     * Returns <tt>true</tt> if this map contains no key-value mappings.
     *
     * @return <tt>true</tt> if this map contains no key-value mappings.
     */
    public boolean isEmpty() {
        return size == 0;
    }

    /**
     * Returns the value to which the specified key is mapped in this identity
     * hash map, or <tt>null</tt> if the map contains no mapping for this key.
     * A return value of <tt>null</tt> does not <i>necessarily</i> indicate
     * that the map contains no mapping for the key; it is also possible that
     * the map explicitly maps the key to <tt>null</tt>. The
     * <tt>containsKey</tt> method may be used to distinguish these two cases.
     *
     * @param   key the key whose associated value is to be returned.
     * @return  the value to which this map maps the specified key, or
     *          <tt>null</tt> if the map contains no mapping for this key.
     * @see #put(Object, Object)
     */
    public V get(Object key) {    	  
        if (key instanceof Integer) return get((Integer)key); /*ibm@117216*/

        int hash = hash(key); /*ibm@98545*/
        int i = indexFor(hash, table.length);
        hash = flagNonIntegerHash(hash);                        /*ibm@121423*/
        Entry<K,V> e = table[i]; 
        
        /*ibm@98538 starts*/
        while (e != null && !(e.hash == hash && eq(key, e.key))) { /*ibm@98545*/
            e = e.next;
        }
        if (e == null)
            return null;
        return e.value;
        /*ibm@98538 ends*/
    }

    /**
     * Specialized private version of get for java.lang.Integer objects
     */    
    /*ibm@117216 starts*/
    private V get(Integer key) {
        int hash = key.intValue();
        int i = indexFor(hash, table.length);
        hash = flagIntegerHash(hash);                          /*ibm@121423*/
        Entry<K,V> e = table[i]; 
        while (e != null && e.hash != hash) { /*ibm@98545*/
            e = e.next;
        }
        if (e == null)
            return null;
        return e.value;
    }
    /*ibm@117216 ends*/

    /**
     * Returns <tt>true</tt> if this map contains a mapping for the
     * specified key.
     *
     * @param   key   The key whose presence in this map is to be tested
     * @return <tt>true</tt> if this map contains a mapping for the specified
     * key.
     */
    public boolean containsKey(Object key) {
        if (key instanceof Integer) return containsKey((Integer)key); /*ibm@117216*/

        int hash = hash(key); /*ibm@98545*/
        int i = indexFor(hash, table.length);
        hash = flagNonIntegerHash(hash);                        /*ibm@121423*/
        Entry e = table[i]; 
        while (e != null) {
            if (e.hash == hash && eq(key, e.key)) /*ibm@98545*/
                return true;
            e = e.next;
        }
        return false;
    }

    /**
     * Specialized private version of containsKey for java.lang.Integer objects
     */    
    /*ibm@117216 starts*/
    private boolean containsKey(Integer key) {
        int hash = key.intValue();
        int i = indexFor(hash, table.length);
        hash = flagIntegerHash(hash);                          /*ibm@121423*/
        Entry e = table[i]; 
        while (e != null) {
            if (e.hash == hash) /*ibm@98545,ibm@117216,ibm@121423*/
                return true;
            e = e.next;
        }
        return false;
    }
    /*ibm@117216 ends*/
    
    /**
     * Returns the entry associated with the specified key in the
     * HashMap.  Returns null if the HashMap contains no mapping
     * for this key.
     */
    Entry<K,V> getEntry(Object key) {
        if (key instanceof Integer) return getEntry((Integer)key); /*ibm@117216*/

        int hash = hash(key); /*ibm@98545*/
        int i = indexFor(hash, table.length);
        hash = flagNonIntegerHash(hash);                        /*ibm@121423*/
        Entry<K,V> e = table[i]; 
        while (e != null && !(e.hash == hash && eq(key, e.key))) /*ibm@98545*/
            e = e.next;
        return e;
    }
  
    /**
     * Specialized private version of getEntry for java.lang.Integer objects
     */    
    /*ibm@117216 starts*/
    private Entry<K,V> getEntry(Integer key) {
        int hash = key.intValue(); /*ibm@98545*/
        int i = indexFor(hash, table.length);
        hash = flagIntegerHash(hash);                          /*ibm@121423*/
        Entry<K,V> e = table[i]; 
        while (e != null && e.hash != hash) /*ibm@98545*/ /*ibm@117216*/
            e = e.next;
        return e;
    }
    /*ibm@117216 ends*/
    
    /**
     * Associates the specified value with the specified key in this map.
     * If the map previously contained a mapping for this key, the old
     * value is replaced.
     *
     * @param key key with which the specified value is to be associated.
     * @param value value to be associated with the specified key.
     * @return previous value associated with specified key, or <tt>null</tt>
     *           if there was no mapping for key.  A <tt>null</tt> return can
     *           also indicate that the HashMap previously associated
     *           <tt>null</tt> with the specified key.
     */
    public V put(K key, V value) {
        if (key instanceof Integer) return put((Integer) key, value); /*ibm@117216*/

        int hash = hash(key); /*ibm@98545*/
        int i = indexFor(hash, table.length);
        hash = flagNonIntegerHash(hash);                        /*ibm@121423*/

        for (Entry<K,V> e = table[i]; e != null; e = e.next) {
            if (e.hash == hash && eq(key, e.key)) { /*ibm@98545*/
                V oldValue = e.value;
                e.value = value;
                e.recordAccess(this);
                return oldValue;
            }
        }

        modCount++;
        addEntry(hash, key, value, i); /*ibm@98545*/
        return null;
    }

    /**
     * Specialized private version of put for java.lang.Integer keys
     */
    /*ibm@117216 starts*/
    private V put(Integer key, V value) {
        int hash = key.intValue();
        int i = indexFor(hash, table.length);
        hash = flagIntegerHash(hash);                          /*ibm@121423*/

        for (Entry<K,V> e = table[i]; e != null; e = e.next) {
            if (e.hash == hash) {                              /*ibm@121423*/
                V oldValue = e.value;
                e.value = value;
                e.recordAccess(this);
                return oldValue;
            }
        }

        modCount++;
        addEntry(hash, (K)key, value, i); /*ibm@98545,ibm@117216,ibm@121423*/
        return null;
    }
    /*ibm@117216 ends*/

    /**
     * This method is used instead of put by constructors and
     * pseudoconstructors (clone, readObject).  It does not resize the table,
     * check for comodification, etc.  It calls createEntry rather than
     * addEntry.
     */
    private void putForCreate(K key, V value) {
    	  /*ibm@117216 starts*/
        if (key instanceof Integer)  {
            putForCreate((Integer) key, value);
        } else {
    	  /*ibm@117216 ends*/
            int hash = hash(key); /*ibm@98545*/
            int i = indexFor(hash, table.length);
            hash = flagNonIntegerHash(hash);                    /*ibm@121423*/
    
            /**
             * Look for preexisting entry for key.  This will never happen for
             * clone or deserialize.  It will only happen for construction if the
             * input Map is a sorted map whose ordering is inconsistent w/ equals.
             */
            for (Entry<K,V> e = table[i]; e != null; e = e.next) {
                if (e.hash == hash && eq(key, e.key)) { /*ibm@98545*/
                    e.value = value;
                    return;
                }
            }
    
            createEntry(hash, key, value, i); /*ibm@98545*/
          }
    }

    /**
     * Specialized private version of putForCreate for java.lang.Integer keys
     */
    /*ibm@117216 starts*/
    private void putForCreate(Integer key, V value) {
        int hash = key.intValue(); /*ibm@98545*/
        int i = indexFor(hash, table.length);
        hash = flagIntegerHash(hash);                          /*ibm@121423*/

        /**
         * Look for preexisting entry for key.  This will never happen for
         * clone or deserialize.  It will only happen for construction if the
         * input Map is a sorted map whose ordering is inconsistent w/ equals.
         */
        for (Entry<K,V> e = table[i]; e != null; e = e.next) {
            if (e.hash == hash) { /*ibm@98545,ibm@117216,ibm@121423*/
                e.value = value;
                return;
            }
        }

        createEntry(hash, (K)key, value, i); /*ibm@98545,ibm@117216,ibm@121423*/
    }
    /*ibm@117216 ends*/

    void putAllForCreate(Map<? extends K, ? extends V> m) {
        for (Iterator<? extends Map.Entry<? extends K, ? extends V>> i = m.entrySet().iterator(); i.hasNext(); ) {
            Map.Entry<? extends K, ? extends V> e = i.next();
            putForCreate(e.getKey(), e.getValue());
        }
    }

    /**
     * Rehashes the contents of this map into a new array with a
     * larger capacity.  This method is called automatically when the
     * number of keys in this map reaches its threshold.
     *
     * If current capacity is MAXIMUM_CAPACITY, this method does not
     * resize the map, but sets threshold to Integer.MAX_VALUE.
     * This has the effect of preventing future calls.
     *
     * @param newCapacity the new capacity, MUST be a power of two;
     *        must be greater than current capacity unless current
     *        capacity is MAXIMUM_CAPACITY (in which case value
     *        is irrelevant).
     */
    void resize(int newCapacity) {
        Entry[] oldTable = table;
        int oldCapacity = oldTable.length;
        if (oldCapacity == MAXIMUM_CAPACITY) {
            threshold = Integer.MAX_VALUE;
            return;
        }

        Entry[] newTable = new Entry[newCapacity];
        transfer(newTable);
        table = newTable;
        threshold = (int)(newCapacity * loadFactor);
    }

    /** 
     * Transfer all entries from current table to newTable.
     */
    void transfer(Entry[] newTable) {
        /*ibm@84391.1 starts */
        onEntry();
        try {
            transfer0(newTable);
        } finally {
            onExit();
        }
    }

    private void transfer0(Entry[] newTable) {
        /*ibm@84391.1 ends */
        Entry[] src = table;
        int newCapacity = newTable.length;
        for (int j = 0; j < src.length; j++) {
            Entry<K,V> e = src[j];
            if (e != null) {
                src[j] = null;
                do {
                    Entry<K,V> next = e.next;
                    int i = indexFor(unFlagHash(e.hash,j), newCapacity);  
                    e.next = newTable[i];
                    newTable[i] = e;
                    e = next;
                } while (e != null);
            }
        }
    }

    /**
     * Copies all of the mappings from the specified map to this map
     * These mappings will replace any mappings that
     * this map had for any of the keys currently in the specified map.
     *
     * @param m mappings to be stored in this map.
     * @throws NullPointerException if the specified map is null.
     */
    public void putAll(Map<? extends K, ? extends V> m) {
        int numKeysToBeAdded = m.size();
        if (numKeysToBeAdded == 0)
            return;

        /*
         * Expand the map if the map if the number of mappings to be added
         * is greater than or equal to threshold.  This is conservative; the
         * obvious condition is (m.size() + size) >= threshold, but this
         * condition could result in a map with twice the appropriate capacity,
         * if the keys to be added overlap with the keys already in this map.
         * By using the conservative calculation, we subject ourself
         * to at most one extra resize.
         */
        if (numKeysToBeAdded > threshold) {
            int targetCapacity = (int)(numKeysToBeAdded / loadFactor + 1);
            if (targetCapacity > MAXIMUM_CAPACITY)
                targetCapacity = MAXIMUM_CAPACITY;
            int newCapacity = table.length;
            while (newCapacity < targetCapacity)
                newCapacity <<= 1;
            if (newCapacity > table.length)
                resize(newCapacity);
        }

        for (Iterator<? extends Map.Entry<? extends K, ? extends V>> i = m.entrySet().iterator(); i.hasNext(); ) {
            Map.Entry<? extends K, ? extends V> e = i.next();
            put(e.getKey(), e.getValue());
        }
    }
  
    /**
     * Removes the mapping for this key from this map if present.
     *
     * @param  key key whose mapping is to be removed from the map.
     * @return previous value associated with specified key, or <tt>null</tt>
     *           if there was no mapping for key.  A <tt>null</tt> return can
     *           also indicate that the map previously associated <tt>null</tt>
     *           with the specified key.
     */
    public V remove(Object key) {
        Entry<K,V> e = removeEntryForKey(key);
        return (e == null ? null : e.value);
    }

    /**
     * Removes and returns the entry associated with the specified key
     * in the HashMap.  Returns null if the HashMap contains no mapping
     * for this key.
     */
    Entry<K,V> removeEntryForKey(Object key) {
        if (key instanceof Integer) return removeEntryForKey((Integer) key); /*ibm@117216*/
        int hash = hash(key); /*ibm@98545*/
        int i = indexFor(hash, table.length);
        hash = flagNonIntegerHash(hash);                        /*ibm@121423*/
        Entry<K,V> prev = table[i];
        Entry<K,V> e = prev;

        while (e != null) {
            Entry<K,V> next = e.next;
            if (e.hash == hash && eq(key, e.key)) { /*ibm@98545*/
                modCount++;
                size--;
                if (prev == e) 
                    table[i] = next;
                else
                    prev.next = next;
                e.recordRemoval(this);
                return e;
            }
            prev = e;
            e = next;
        }
   
        return e;
    }

							/*ibm@117216 start*/
    private Entry<K,V> removeEntryForKey(Integer key) {
        int hash = key.intValue(); /*ibm@117216*/
        int i = indexFor(hash, table.length);
        hash = flagIntegerHash(hash);                          /*ibm@121423*/
        Entry<K,V> prev = table[i];
        Entry<K,V> e = prev;

        while (e != null) {
            Entry<K,V> next = e.next;
            if (e.hash == hash) { /*ibm@117216,ibm@121423*/
                modCount++;
                size--;
                if (prev == e) 
                    table[i] = next;
                else
                    prev.next = next;
                e.recordRemoval(this);
                return e;
            }
            prev = e;
            e = next;
        }
   
        return e;
    }
							/*ibm@117216 end*/

    /**
     * Special version of remove for EntrySet.
     */
    Entry<K,V> removeMapping(Object o) {
        if (!(o instanceof Map.Entry))
            return null;

        Map.Entry<K,V> entry = (Map.Entry<K,V>) o;
        Object k = entry.getKey(); /*ibm@98545*/

        if (k instanceof Integer) return removeMappingInt(o); /*ibm@117216*/
                
        int hash = hash(k);
        int i = indexFor(hash, table.length);
        hash = flagNonIntegerHash(hash);                        /*ibm@121423*/
        Entry<K,V> prev = table[i];
        Entry<K,V> e = prev;

        while (e != null) {
            Entry<K,V> next = e.next;
            if (e.hash == hash && e.equals(entry)) {
                modCount++;
                size--;
                if (prev == e) 
                    table[i] = next;
                else
                    prev.next = next;
                e.recordRemoval(this);
                return e;
            }
            prev = e;
            e = next;
        }
   
        return e;
    }

							/*ibm@117216 start*/
    private Entry<K,V> removeMappingInt(Object o) {
        Map.Entry<K,V> entry = (Map.Entry<K,V>) o;
        Integer k = (Integer)entry.getKey(); /*ibm@117216*/
        int hash = k.intValue();
        int i = indexFor(hash, table.length);                  /*ibm@121735*/
        hash = flagIntegerHash(hash);                          /*ibm@121423*/
        Entry<K,V> prev = table[i];
        Entry<K,V> e = prev;

        while (e != null) {
            Entry<K,V> next = e.next;
            if (e.hash == hash) {                             /*ibm@121423*/
                modCount++;
                size--;
                if (prev == e) 
                    table[i] = next;
                else
                    prev.next = next;
                e.recordRemoval(this);
                return e;
            }
            prev = e;
            e = next;
        }
   
        return e;
    }
							/*ibm@117216 end*/

    /**
     * Removes all mappings from this map.
     */
    public void clear() {
        modCount++;
        Entry[] tab = table;
        for (int i = 0; i < tab.length; i++) 
            tab[i] = null;
        size = 0;
    }

    /**
     * Returns <tt>true</tt> if this map maps one or more keys to the
     * specified value.
     *
     * @param value value whose presence in this map is to be tested.
     * @return <tt>true</tt> if this map maps one or more keys to the
     *         specified value.
     */
    public boolean containsValue(Object value) {
        if (value == null) 
            return containsNullValue();

        Entry[] tab = table;
        for (int i = 0; i < tab.length ; i++)
            for (Entry e = tab[i] ; e != null ; e = e.next)
                if (value.equals(e.value))
                    return true;
        return false;
    }

    /**
     * Special-case code for containsValue with null argument
     **/
    private boolean containsNullValue() {
        Entry[] tab = table;
        for (int i = 0; i < tab.length ; i++)
            for (Entry e = tab[i] ; e != null ; e = e.next)
                if (e.value == null)
                    return true;
        return false;
    }

    /**
     * Returns a shallow copy of this <tt>HashMap</tt> instance: the keys and
     * values themselves are not cloned.
     *
     * @return a shallow copy of this map.
     */
    public Object clone() {
        HashMap<K,V> result = null;
    
        try { 
            result = (HashMap<K,V>)super.clone();
        } catch (CloneNotSupportedException e) { 
           // assert false;
        }
        result.table = new Entry[table.length];
        result.entrySet = null;
        result.modCount = 0;
        result.size = 0;
        result.init();
        result.putAllForCreate(this);

        return result;
    }

    static class Entry<K,V> implements Map.Entry<K,V> {
        final K key;
        V value;
        final int hash;
        Entry<K,V> next;

        /**
         * Create new entry.
         */
        Entry(int h, K k, V v, Entry<K,V> n) {
            value = v;
            next = n;
            key = k;
            hash = h;
        }

        public K getKey() {
             return key; /*ibm@98545*/
        }

        public V getValue() {
            return value;
        }
    
        public V setValue(V newValue) {
            V oldValue = value;
            value = newValue;
            return oldValue;
        }
    
        public boolean equals(Object o) {
            if (!(o instanceof Map.Entry))
                return false;
            Map.Entry e = (Map.Entry)o;
            Object k1 = getKey();
            Object k2 = e.getKey();
            /*ibm@117216 starts*/
            int hash2 = hash;
            if (k2 instanceof Integer) {
                hash2 = ((Integer)k2).intValue();
                hash2 = flagIntegerHash(hash2);           /* ibm@124312 */
            }
            /*ibm@117216 ends*/
            
            if (k1 == k2 || (k1 != null && k1.equals(k2) && hash == hash2)) { /*ibm@117216*/
                Object v1 = getValue();
                Object v2 = e.getValue();
                if (v1 == v2 || (v1 != null && v1.equals(v2))) 
                    return true;
            }
            return false;
        }
    
        public int hashCode() {
            return (key==null ? 0 : getKey().hashCode()) ^ /*ibm@117216*/
                   (value==null   ? 0 : value.hashCode()); /*ibm@98545*/
        }
    
        public String toString() {
            return getKey() + "=" + getValue();
        }

        /**
         * This method is invoked whenever the value in an entry is
         * overwritten by an invocation of put(k,v) for a key k that's already
         * in the HashMap.
         */
        void recordAccess(HashMap<K,V> m) {
        }

        /**
         * This method is invoked whenever the entry is
         * removed from the table.
         */
        void recordRemoval(HashMap<K,V> m) {
        }
    }

    /**
     * Add a new entry with the specified key, value and hash code to
     * the specified bucket.  It is the responsibility of this 
     * method to resize the table if appropriate.
     *
     * Subclass overrides this to alter the behavior of put method.
     */
    void addEntry(int hash, K key, V value, int bucketIndex) {
        Entry<K,V> e = table[bucketIndex];
        table[bucketIndex] = new Entry<K,V>(hash, key, value, e);
        if (size++ >= threshold)
            resize(2 * table.length);
    }

    /**
     * Like addEntry except that this version is used when creating entries
     * as part of Map construction or "pseudo-construction" (cloning,
     * deserialization).  This version needn't worry about resizing the table.
     *
     * Subclass overrides this to alter the behavior of HashMap(Map),
     * clone, and readObject.
     */
    void createEntry(int hash, K key, V value, int bucketIndex) {
        Entry<K,V> e = table[bucketIndex];
        table[bucketIndex] = new Entry<K,V>(hash, key, value, e);
        size++;
    }

    private abstract class HashIterator<E> implements Iterator<E> {
        Entry<K,V> next;    // next entry to return
        int expectedModCount;    // For fast-fail 
        int index;        // current slot 
        Entry<K,V> current;    // current entry

        HashIterator() {
            expectedModCount = modCount;
            Entry[] t = table;
            int i = t.length;
            Entry<K,V> n = null;
            if (size != 0) { // advance to first entry
                while (i > 0 && (n = t[--i]) == null)
                    ;
            }
            next = n;
            index = i;
        }

        public boolean hasNext() {
            return next != null;
        }

        Entry<K,V> nextEntry() { 
            if (modCount != expectedModCount)
                throw new ConcurrentModificationException();
            Entry<K,V> e = next;
            if (e == null) 
                throw new NoSuchElementException();
                
            Entry<K,V> n = e.next;
            Entry[] t = table;
            int i = index;
            while (n == null && i > 0)
                n = t[--i];
            index = i;
            next = n;
            return current = e;
        }

        public void remove() {
            if (current == null)
                throw new IllegalStateException();
            if (modCount != expectedModCount)
                throw new ConcurrentModificationException();
            /*Use the Entry get method instead of direct access to the key ibm@117216 */
            Object k = current.getKey(); /*ibm@117216*/
            current = null;
            HashMap.this.removeEntryForKey(k);
            expectedModCount = modCount;
        }

    }

    private class ValueIterator extends HashIterator<V> {
        public V next() {
            return nextEntry().value;
        }
    }

    private class KeyIterator extends HashIterator<K> {
        public K next() {
            return nextEntry().getKey();
        }
    }

    private class EntryIterator extends HashIterator<Map.Entry<K,V>> {
        public Map.Entry<K,V> next() {
            return nextEntry();
        }
    }

    // Subclass overrides these to alter behavior of views' iterator() method
    Iterator<K> newKeyIterator()   {
        return new KeyIterator();
    }
    Iterator<V> newValueIterator()   {
        return new ValueIterator();
    }
    Iterator<Map.Entry<K,V>> newEntryIterator()   {
        return new EntryIterator();
    }


    // Views

    private transient Set<Map.Entry<K,V>> entrySet = null;

    /**
     * Returns a set view of the keys contained in this map.  The set is
     * backed by the map, so changes to the map are reflected in the set, and
     * vice-versa.  The set supports element removal, which removes the
     * corresponding mapping from this map, via the <tt>Iterator.remove</tt>,
     * <tt>Set.remove</tt>, <tt>removeAll</tt>, <tt>retainAll</tt>, and
     * <tt>clear</tt> operations.  It does not support the <tt>add</tt> or
     * <tt>addAll</tt> operations.
     *
     * @return a set view of the keys contained in this map.
     */
    public Set<K> keySet() {
        Set<K> ks = keySet;
        return (ks != null ? ks : (keySet = new KeySet()));
    }

    private class KeySet extends AbstractSet<K> {
        public Iterator<K> iterator() {
            return newKeyIterator();
        }
        public int size() {
            return size;
        }
        public boolean contains(Object o) {
            return containsKey(o);
        }
        public boolean remove(Object o) {
            return HashMap.this.removeEntryForKey(o) != null;
        }
        public void clear() {
            HashMap.this.clear();
        }
    }

    /**
     * Returns a collection view of the values contained in this map.  The
     * collection is backed by the map, so changes to the map are reflected in
     * the collection, and vice-versa.  The collection supports element
     * removal, which removes the corresponding mapping from this map, via the
     * <tt>Iterator.remove</tt>, <tt>Collection.remove</tt>,
     * <tt>removeAll</tt>, <tt>retainAll</tt>, and <tt>clear</tt> operations.
     * It does not support the <tt>add</tt> or <tt>addAll</tt> operations.
     *
     * @return a collection view of the values contained in this map.
     */
    public Collection<V> values() {
        Collection<V> vs = values;
        return (vs != null ? vs : (values = new Values()));
    }

    private class Values extends AbstractCollection<V> {
        public Iterator<V> iterator() {
            return newValueIterator();
        }
        public int size() {
            return size;
        }
        public boolean contains(Object o) {
            return containsValue(o);
        }
        public void clear() {
            HashMap.this.clear();
        }
    }

    /**
     * Returns a collection view of the mappings contained in this map.  Each
     * element in the returned collection is a <tt>Map.Entry</tt>.  The
     * collection is backed by the map, so changes to the map are reflected in
     * the collection, and vice-versa.  The collection supports element
     * removal, which removes the corresponding mapping from the map, via the
     * <tt>Iterator.remove</tt>, <tt>Collection.remove</tt>,
     * <tt>removeAll</tt>, <tt>retainAll</tt>, and <tt>clear</tt> operations.
     * It does not support the <tt>add</tt> or <tt>addAll</tt> operations.
     *
     * @return a collection view of the mappings contained in this map.
     * @see Map.Entry
     */
    public Set<Map.Entry<K,V>> entrySet() {
        Set<Map.Entry<K,V>> es = entrySet;
        return (es != null ? es : (entrySet = (Set<Map.Entry<K,V>>) (Set) new EntrySet()));
    }

    private class EntrySet extends AbstractSet/*<Map.Entry<K,V>>*/ {
        public Iterator/*<Map.Entry<K,V>>*/ iterator() {
            return newEntryIterator();
        }
        public boolean contains(Object o) {
            if (!(o instanceof Map.Entry))
                return false;
            Map.Entry<K,V> e = (Map.Entry<K,V>) o;
            Entry<K,V> candidate = getEntry(e.getKey());
            return candidate != null && candidate.equals(e);
        }
        public boolean remove(Object o) {
            return removeMapping(o) != null;
        }
        public int size() {
            return size;
        }
        public void clear() {
            HashMap.this.clear();
        }
    }

    /**
     * Save the state of the <tt>HashMap</tt> instance to a stream (i.e.,
     * serialize it).
     *
     * @serialData The <i>capacity</i> of the HashMap (the length of the
     *           bucket array) is emitted (int), followed  by the
     *           <i>size</i> of the HashMap (the number of key-value
     *           mappings), followed by the key (Object) and value (Object)
     *           for each key-value mapping represented by the HashMap
     *             The key-value mappings are emitted in the order that they
     *             are returned by <tt>entrySet().iterator()</tt>.
     * 
     */
    private void writeObject(java.io.ObjectOutputStream s)
        throws IOException
    {
        Iterator<Map.Entry<K,V>> i = entrySet().iterator();
     
        // Write out the threshold, loadfactor, and any hidden stuff
        s.defaultWriteObject();

        // Write out number of buckets
        s.writeInt(table.length);

        // Write out size (number of Mappings)
        s.writeInt(size);

        // Write out keys and values (alternating)
        while (i.hasNext()) {
            Map.Entry<K,V> e = i.next();
            s.writeObject(e.getKey());
            s.writeObject(e.getValue());
        }
    }

    private static final long serialVersionUID = 362498820763181265L;

    /**
     * Reconstitute the <tt>HashMap</tt> instance from a stream (i.e.,
     * deserialize it).
     */
    private void readObject(java.io.ObjectInputStream s)
         throws IOException, ClassNotFoundException
    {
        // Read in the threshold, loadfactor, and any hidden stuff
        s.defaultReadObject();

        // Read in number of buckets and allocate the bucket array;
        int numBuckets = s.readInt();
        table = new Entry[numBuckets];

        init();  // Give subclass a chance to do its thing.

        // Read in size (number of Mappings)
        int size = s.readInt();

        // Read the keys and values, and put the mappings in the HashMap
        for (int i=0; i<size; i++) {
            K key = (K) s.readObject();
            V value = (V) s.readObject();
            putForCreate(key, value);
        }
    }

    // These methods are used when serializing HashSets
    int   capacity()     { return table.length; }
    float loadFactor()   { return loadFactor;   }
}
